<?php

namespace Litecms\Blacklist\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Blacklist\Http\Requests\BlacklistRequest;
use Litecms\Blacklist\Interfaces\BlacklistRepositoryInterface;
use Litecms\Blacklist\Models\Blacklist;

/**
 * Resource controller class for blacklist.
 */
class BlacklistResourceController extends BaseController
{

    /**
     * Initialize blacklist resource controller.
     *
     * @param type BlacklistRepositoryInterface $blacklist
     *
     * @return null
     */
    public function __construct(BlacklistRepositoryInterface $blacklist)
    {
        parent::__construct();
        $this->repository = $blacklist;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Blacklist\Repositories\Criteria\BlacklistResourceCriteria::class);
    }

    /**
     * Display a list of blacklist.
     *
     * @return Response
     */
    public function index(BlacklistRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Blacklist\Repositories\Presenter\BlacklistPresenter::class)
                ->$function();
        }

        $blacklists = $this->repository->paginate();

        return $this->response->title(trans('blacklist::blacklist.names'))
            ->view('blacklist::blacklist.index', true)
            ->data(compact('blacklists'))
            ->output();
    }

    /**
     * Display blacklist.
     *
     * @param Request $request
     * @param Model   $blacklist
     *
     * @return Response
     */
    public function show(BlacklistRequest $request, Blacklist $blacklist)
    {

        if ($blacklist->exists) {
            $view = 'blacklist::blacklist.show';
        } else {
            $view = 'blacklist::blacklist.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('blacklist::blacklist.name'))
            ->data(compact('blacklist'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new blacklist.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BlacklistRequest $request)
    {

        $blacklist = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('blacklist::blacklist.name')) 
            ->view('blacklist::blacklist.create', true) 
            ->data(compact('blacklist'))
            ->output();
    }

    /**
     * Create new blacklist.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BlacklistRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $blacklist                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('blacklist::blacklist.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('blacklist/blacklist/' . $blacklist->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/blacklist/blacklist'))
                ->redirect();
        }

    }

    /**
     * Show blacklist for editing.
     *
     * @param Request $request
     * @param Model   $blacklist
     *
     * @return Response
     */
    public function edit(BlacklistRequest $request, Blacklist $blacklist)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('blacklist::blacklist.name'))
            ->view('blacklist::blacklist.edit', true)
            ->data(compact('blacklist'))
            ->output();
    }

    /**
     * Update the blacklist.
     *
     * @param Request $request
     * @param Model   $blacklist
     *
     * @return Response
     */
    public function update(BlacklistRequest $request, Blacklist $blacklist)
    {
        try {
            $attributes = $request->all();

            $blacklist->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('blacklist::blacklist.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('blacklist/blacklist/' . $blacklist->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('blacklist/blacklist/' . $blacklist->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the blacklist.
     *
     * @param Model   $blacklist
     *
     * @return Response
     */
    public function destroy(BlacklistRequest $request, Blacklist $blacklist)
    {
        try {

            $blacklist->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('blacklist::blacklist.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('blacklist/blacklist/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('blacklist/blacklist/' . $blacklist->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple blacklist.
     *
     * @param Model   $blacklist
     *
     * @return Response
     */
    public function delete(BlacklistRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('blacklist::blacklist.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('blacklist/blacklist'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/blacklist/blacklist'))
                ->redirect();
        }

    }

    /**
     * Restore deleted blacklists.
     *
     * @param Model   $blacklist
     *
     * @return Response
     */
    public function restore(BlacklistRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('blacklist::blacklist.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/blacklist/blacklist'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/blacklist/blacklist/'))
                ->redirect();
        }

    }

}
